
#ifndef _temporary_hacks_h
#define _temporary_hacks_h

#ifndef __KERNEL__

#define mem_vaddr unsigned long
#define mem_addr unsigned long
#define io_vaddr unsigned long
#define io_addr unsigned long
#define	IO_NULL		((io_vaddr) 0)
#define	MEM_NULL		((mem_vaddr) 0)

#include <kgi/kgi.h>

struct io_region {						  
	io_vaddr baseptr;	/* virtual base address		*/
	io_addr base;		/* physical base address	*/
        io_addr size;		/* size of region		*/
	io_addr decode;	        /* decoded phys address lines	*/
	char *name;			/* name of the region		*/
};


struct mem_region {						  
	mem_vaddr baseptr;	/* virtual base address		*/
	mem_addr base;		/* physical base address	*/
	mem_addr size;		/* size of region		*/
	mem_addr decode;	/* decoded phys address lines	*/
	char *name;			/* name of the region		*/
};

static inline kgiu8 io_in8(io_vaddr addr) 
{
	return inb(addr);
}

static inline kgiu16 io_in16(io_vaddr addr)
{
	return inw(addr);
}

static inline kgiu32 io_in32(io_vaddr addr)
{
	return inl(addr);
}

static inline void io_ins8(io_vaddr addr, void *buf, kgi_uint cnt)
{
	insb(addr, buf, cnt);
}

static inline void io_ins16(io_vaddr addr, void *buf, kgi_uint cnt)
{
	insw(addr, buf, cnt);
}

static inline void io_ins32(io_vaddr addr, void *buf, kgi_uint cnt)
{
	insl(addr, buf, cnt);
}

static inline void io_out8(kgiu8 val, io_vaddr addr)
{
	outb(val, addr);
}

static inline void io_out16(kgiu16 val, io_vaddr addr)
{
	outw(val, addr);
}

static inline void io_out32(kgiu32 val, io_vaddr addr)
{
	outl(val, addr);
}

static inline void io_outs8(io_vaddr addr, void *buf, kgi_uint cnt)
{
	outsb(addr, buf, cnt);
}

static inline void io_outs16(io_vaddr addr, void *buf, kgi_uint cnt)
{
	outsw(addr, buf, cnt);
}

static inline void io_outs32(io_vaddr addr, void *buf, kgi_uint cnt)
{
	outsl(addr, buf, cnt);
}

static inline kgiu8 mem_in8(mem_vaddr addr)
{
	return 	readb(addr);
}

static inline kgiu16 mem_in16(mem_vaddr addr)
{
	return readw(addr);
}

static inline kgiu32 mem_in32(mem_vaddr addr)
{
	return readl(addr);
}

static inline void mem_out8(kgiu8 val, mem_vaddr addr)
{
	writeb(val, addr);
}

static inline void mem_out16(kgiu16 val, mem_vaddr addr)
{
	writew(val, addr);
}

static inline void mem_out32(kgiu32 val, mem_vaddr addr)
{
	writel(val, addr);
}

#define	MEM_INS(x) \
static inline void mem_ins##x(mem_vaddr addr, uint##x *buf, kgi_uint cnt)	\
{								\
	while (cnt--) {						\
								\
		*(buf++) = mem_in##x(addr);			\
	}							\
}

#define	MEM_OUTS(x) \
static inline void mem_outs##x(mem_vaddr addr, uint##x *buf, kgi_uint cnt)	\
{								\
	while (cnt--) {						\
								\
		mem_out##x(*(buf++), addr);			\
	}							\
}

MEM_INS(8)
MEM_INS(16)
MEM_INS(32)

MEM_OUTS(8)
MEM_OUTS(16)
MEM_OUTS(32)

#else
#include <kgi/kgi.h>
#endif /* not __KERNEL__ */

#endif /* not _temporary_hacks_h */
